import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/const.dart';
import 'package:flutter_custom_form_component/widget/field/box_decorations.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/money/formatter.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/widget.dart';
import 'package:flutter_custom_form_component/widget/field/text_styles.dart';

const bool moneyFieldUsingPeriodInThousand = true;

/// Text field supports money format.
class MoneyFieldWidget extends OneValueFieldWidget {
  late final String _decimalSeparator, _thousandSeparator;
  late final String? _symbol;

  MoneyFieldWidget(
      {final TextInputAction action = fieldAction,
      final BoxDecorations? decorations,
      final bool enabled = fieldEnabled,
      final String? hint,
      final double horizontalPadding = fieldHorizontalPadding,
      final double? initialValue,
      final Key? key,
      final Function(dynamic value)? onChanged,
      final String? symbol,
      final TextStyles? styles,
      final bool usingPeriodInThousand = moneyFieldUsingPeriodInThousand})
      : super(
            action: action,
            decorations: decorations,
            enabled: enabled,
            hint: hint,
            horizontalPadding: horizontalPadding,
            initialValue: initialValue,
            key: key,
            onChanged: onChanged,
            styles: styles) {
    _symbol = symbol;

    /// Using '.' for separating thousands and ',' for separating decimal. Or the opposite.
    _decimalSeparator = usingPeriodInThousand ? ',' : '.';
    _thousandSeparator = usingPeriodInThousand ? '.' : ',';
  }

  @override
  State<StatefulWidget> createState() => _State();
}

class _State extends State<MoneyFieldWidget>
    with FieldMixin, OneValueFieldMixin {
  final FocusNode _focusNode = FocusNode();
  late final MoneyFormatter _formatter;

  @override
  Widget build(final BuildContext context) => container(TextField(
      controller: controller,
      decoration: InputDecoration(
          border: InputBorder.none,
          hintStyle: widget.styles.hint,
          hintText: widget.hint),
      enabled: widget.enabled,
      focusNode: _focusNode,
      inputFormatters: [
        FilteringTextInputFormatter.allow(
            RegExp('[0-9${widget._decimalSeparator}]')),
        _formatter
      ],
      keyboardType: Platform.isAndroid
          ? const TextInputType.numberWithOptions(decimal: true)
          : TextInputType.text,
      style: style,
      textInputAction: widget.action));

  @override
  void dispose() {
    _focusNode.dispose();
    disposeMixin();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    _formatter = MoneyFormatter(
        decimalSeparator: widget._decimalSeparator,
        symbol: widget._symbol,
        thousandSeparator: widget._thousandSeparator);
    initStateMixin(
        initialValue: widget.initialValue, onChanged: widget.onChanged);
  }

  @override
  String format(final dynamic value) {
    if (value == null) {
      return '';
    }

    /// Replace common decimal separator with widget._decimalSeparator
    String string =
        value!.toString().replaceFirst('.', widget._decimalSeparator);
    final List<String> strings = string.split(widget._decimalSeparator);
    if (int.tryParse(strings[1]) == 0) {
      string = strings[0];
    }
    return _formatter.format(string);
  }

  @override
  BoxDecoration get decoration {
    if (_focusNode.hasFocus) {
      return widget.decorations.active;
    }
    if (valid) {
      return widget.decorations.box;
    }
    return widget.decorations.error;
  }

  @override
  double get horizontalPadding => widget.horizontalPadding;

  @override
  TextStyle get style {
    if (_focusNode.hasFocus) {
      return widget.styles.active;
    }
    if (valid) {
      return widget.styles.text;
    }
    return widget.styles.error;
  }

  @override
  double get value {
    String string = controller.text
        .replaceAll(widget._thousandSeparator, '')
        .replaceFirst(widget._decimalSeparator, '.');
    if (widget._symbol != null) {
      string = string.replaceFirst(widget._symbol!, '');
    }
    return double.tryParse(string) ?? 0.0;
  }
}
